@@ -84,9 +84,11 @@ struct _BubblePrivate {
 	gint             future_height;
 	gboolean         prevent_fade;
 
-	// these will be replaced by notification_t* later on
+	// these will be replaced by class Notification later on
 	GString*         title;
 	GString*         message_body;
+	gboolean         title_needs_refresh;
+	gboolean         message_body_needs_refresh;
 	guint            id;
 	GdkPixbuf*       icon_pixbuf;
 	gint             value; // "empty": -2, valid range: -1..101, -1/101 trigger "over/undershoot"-effect
@@ -94,6 +96,10 @@ struct _BubblePrivate {
 	guint            timeout;
 	guint            urgency;
 	//notification_t* notification;
+
+	// used to prevent unneeded updates of the tile-cache, for append-,
+	// update or replace-cases, needs to move into class Notification
+	GString*         old_icon_filename;
 };
 
 enum
@@ -727,7 +733,7 @@ _refresh_background (Bubble* self)
 	cairo_surface_destroy (dummy);
 
 	// now blur the surface-clone
-	blur = raico_blur_create (RAICO_BLUR_QUALITY_HIGH);
+	blur = raico_blur_create (RAICO_BLUR_QUALITY_LOW);
 	raico_blur_set_radius (blur, BUBBLE_CONTENT_BLUR_RADIUS);
 	raico_blur_apply (blur, clone);
 	raico_blur_destroy (blur);
@@ -928,7 +934,7 @@ _refresh_icon (Bubble* self)
 	// create the surface/blur-cache from the normal surface
 	if (priv->tile_icon)
 		tile_destroy (priv->tile_icon);
-	priv->tile_icon = tile_new (normal, BUBBLE_CONTENT_BLUR_RADIUS);
+	priv->tile_icon = tile_new (normal, BUBBLE_CONTENT_BLUR_RADIUS/2);
 
 	// clean up
 	cairo_destroy (cr);
@@ -1015,7 +1021,7 @@ _refresh_title (Bubble* self)
 	pango_cairo_show_layout (cr, layout);
 
 	// ... blur it
-	blur = raico_blur_create (RAICO_BLUR_QUALITY_HIGH);
+	blur = raico_blur_create (RAICO_BLUR_QUALITY_LOW);
 	raico_blur_set_radius (blur, TEXT_DROP_SHADOW_SIZE);
 	raico_blur_apply (blur, normal);
 	raico_blur_destroy (blur);
@@ -1038,6 +1044,9 @@ _refresh_title (Bubble* self)
 	// clean up
 	cairo_destroy (cr);
 	cairo_surface_destroy (normal);
+
+	// reset refresh-flag
+	priv->title_needs_refresh = FALSE;
 }
 
 void
@@ -1122,7 +1131,7 @@ _refresh_body (Bubble* self)
 	pango_cairo_show_layout (cr, layout);
 
 	// ... blur it
-	blur = raico_blur_create (RAICO_BLUR_QUALITY_HIGH);
+	blur = raico_blur_create (RAICO_BLUR_QUALITY_LOW);
 	raico_blur_set_radius (blur, TEXT_DROP_SHADOW_SIZE);
 	raico_blur_apply (blur, normal);
 	raico_blur_destroy (blur);
@@ -1145,6 +1154,9 @@ _refresh_body (Bubble* self)
 	// clean up
 	cairo_destroy (cr);
 	cairo_surface_destroy (normal);
+
+	// reset refresh-flag
+	priv->message_body_needs_refresh = FALSE;
 }
 
 void
@@ -1195,7 +1207,7 @@ _refresh_indicator (Bubble* self)
 	// create the surface/blur-cache from the normal surface
 	if (priv->tile_indicator)
 		tile_destroy (priv->tile_indicator);
-	priv->tile_indicator = tile_new (normal, BUBBLE_CONTENT_BLUR_RADIUS);
+	priv->tile_indicator = tile_new (normal, BUBBLE_CONTENT_BLUR_RADIUS/2);
 
 	// clean up
 	cairo_destroy (cr);
@@ -1579,48 +1591,18 @@ screen_changed_handler (GtkWidget* window,
 
 static
 void
-update_input_shape (GtkWidget* window,
-		    gint       width,
-		    gint       height)
+update_input_shape (GtkWidget* window)
 {
-	GdkBitmap* mask = NULL;
-	cairo_t*   cr   = NULL;
+	GdkRegion*   region = NULL;
 
-	mask = (GdkBitmap*) gdk_pixmap_new (NULL, width, height, 1);
-	if (mask)
-	{
-		cr = gdk_cairo_create (mask);
-		if (cairo_status (cr) == CAIRO_STATUS_SUCCESS)
-		{
-			cairo_scale (cr, 1.0f, 1.0f);
-			cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
-			cairo_paint (cr);
-			cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
-			cairo_set_source_rgb (cr, 1.0f, 1.0f, 1.0f);
+	// sanity check
+	if (!window)
+		return;
 
-			// just draw something
-			draw_round_rect (cr,
-					 1.0f,
-					 0.0f, 0.0f,
-					 10.0f,
-					 10.0f, 10.0f);
-
-			cairo_fill (cr);
-
-			cairo_destroy (cr);
-
-			gtk_widget_input_shape_combine_mask (window,
-							     NULL,
-							     0,
-							     0);
-			gtk_widget_input_shape_combine_mask (window,
-							     mask,
-							     0,
-							     0);
-		}
-
-		g_object_unref ((gpointer) mask);
-	}
+	// set an empty input-mask to allow click-through 
+	region = gdk_region_new ();
+	gdk_window_input_shape_combine_region (window->window, region, 0, 0);
+	gdk_region_destroy (region);
 }
 
 static void
@@ -1647,61 +1629,57 @@ update_shape (Bubble* self)
 		return;
 	}
 
-	// guess we need one
-	gtk_widget_get_size_request (priv->widget, &width, &height);
-	mask = (GdkBitmap*) gdk_pixmap_new (NULL, width, height, 1);
-	if (mask)
+	// we're not-composited, so deal with mouse-over differently
+	if (bubble_is_mouse_over (self))
 	{
-		// create context from mask/pixmap
-		cr = gdk_cairo_create (mask);
-		if (cairo_status (cr) == CAIRO_STATUS_SUCCESS)
+		gtk_widget_hide (priv->widget);
+	}
+	else
+	{
+		gtk_widget_show (priv->widget);
+		gtk_widget_get_size_request (priv->widget, &width, &height);
+		mask = (GdkBitmap*) gdk_pixmap_new (NULL, width, height, 1);
+		if (mask)
 		{
-			// clear mask/context
-			cairo_scale (cr, 1.0f, 1.0f);
-			cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
-			cairo_paint (cr);
-
-			width  -= 2 * EM2PIXELS (defaults_get_bubble_shadow_size (d), d);
-			height -= 2 * EM2PIXELS (defaults_get_bubble_shadow_size (d), d);
-
-			// draw rounded rectangle shape/mask
-			if (bubble_is_mouse_over (self))
+			// create context from mask/pixmap
+			cr = gdk_cairo_create (mask);
+			if (cairo_status (cr) == CAIRO_STATUS_SUCCESS)
+			{
+				// clear mask/context
+				cairo_scale (cr, 1.0f, 1.0f);
 				cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
-			else
+				cairo_paint (cr);
+
+				width  -= 2 * EM2PIXELS (defaults_get_bubble_shadow_size (d), d);
+				height -= 2 * EM2PIXELS (defaults_get_bubble_shadow_size (d), d);
+
+				// draw rounded rectangle shape/mask
 				cairo_set_operator (cr, CAIRO_OPERATOR_OVER);
-			cairo_set_source_rgb (cr, 1.0f, 1.0f, 1.0f);
-			draw_round_rect (cr,
+				cairo_set_source_rgb (cr, 1.0f, 1.0f, 1.0f);
+				draw_round_rect (cr,
 					 1.0f,
 					 EM2PIXELS (defaults_get_bubble_shadow_size (d), d),
 					 EM2PIXELS (defaults_get_bubble_shadow_size (d), d),
 					 EM2PIXELS (defaults_get_bubble_corner_radius (d), d),
 					 width,
 					 height);
-			cairo_fill (cr);
-			/*if (bubble_is_mouse_over (self))
-			{
-				cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
-				draw_round_rect (
-					cr,
-					1.0f,
-					2 + EM2PIXELS (defaults_get_bubble_shadow_size (d), d),
-					2 + EM2PIXELS (defaults_get_bubble_shadow_size (d), d),
-					EM2PIXELS (defaults_get_bubble_corner_radius (d), d),
-					width - 4,
-					height - 4);
 				cairo_fill (cr);
-			}*/
+				cairo_destroy (cr);
 
-			cairo_destroy (cr);
+				// remove any current shape-mask
+				gtk_widget_shape_combine_mask (priv->widget,
+							       NULL,
+							       0,
+							       0);
 
-			// remove any current shape-mask
-			gtk_widget_shape_combine_mask (priv->widget,NULL, 0, 0);
-
-			// set new shape-mask
-			gtk_widget_shape_combine_mask (priv->widget,mask, 0, 0);
+				// set new shape-mask
+				gtk_widget_shape_combine_mask (priv->widget,
+							       mask,
+							       0,
+							       0);
+			}
+			g_object_unref ((gpointer) mask);
 		}
-
-		g_object_unref ((gpointer) mask);
 	}
 }
 
@@ -2077,6 +2055,12 @@ bubble_dispose (GObject* gobject)
 		priv->tile_indicator = NULL;
 	}
 
+	if (priv->old_icon_filename)
+	{
+		g_string_free ((gpointer) priv->old_icon_filename, TRUE);
+		priv->old_icon_filename = NULL;
+	}
+
 	// chain up to the parent class
 	G_OBJECT_CLASS (bubble_parent_class)->dispose (gobject);
 }
@@ -2097,17 +2081,19 @@ bubble_init (Bubble* self)
 
 	BubblePrivate *priv;
 
-	self->priv = priv       = GET_PRIVATE (self);
-	priv->layout            = LAYOUT_NONE;
-	priv->title             = NULL;
-	priv->message_body      = NULL;
-	priv->visible           = FALSE;
-	priv->icon_pixbuf       = NULL;
-	priv->value             = -2;
-	priv->synchronous       = NULL;
-	priv->sender            = NULL;
-	priv->draw_handler_id   = 0;
-	priv->pointer_update_id = 0;
+	self->priv = priv                = GET_PRIVATE (self);
+	priv->layout                     = LAYOUT_NONE;
+	priv->title                      = NULL;
+	priv->message_body               = NULL;
+	priv->title_needs_refresh        = FALSE;
+	priv->message_body_needs_refresh = FALSE;
+	priv->visible                    = FALSE;
+	priv->icon_pixbuf                = NULL;
+	priv->value                      = -2;
+	priv->synchronous                = NULL;
+	priv->sender                     = NULL;
+	priv->draw_handler_id            = 0;
+	priv->pointer_update_id          = 0;
 }
 
 static void
@@ -2256,35 +2242,38 @@ bubble_new (Defaults* defaults)
 
 	// TODO: fold some of that back into bubble_init
 	this->priv = GET_PRIVATE (this);
-	this->priv->layout               = LAYOUT_NONE;
-	this->priv->widget               = window;
-	this->priv->title                = g_string_new ("");
-	this->priv->message_body         = NULL;
-	this->priv->icon_pixbuf          = NULL;
-	this->priv->value                = -2;
-	this->priv->visible              = FALSE;
-	this->priv->timeout              = 5000;
-	this->priv->mouse_over           = FALSE;
-	this->priv->distance             = 1.0f;
-	this->priv->composited           = gdk_screen_is_composited (
+	this->priv->layout                     = LAYOUT_NONE;
+	this->priv->widget                     = window;
+	this->priv->title                      = g_string_new ("");
+	this->priv->message_body               = g_string_new ("");
+	this->priv->title_needs_refresh        = FALSE;
+	this->priv->message_body_needs_refresh = FALSE;
+	this->priv->icon_pixbuf                = NULL;
+	this->priv->value                      = -2;
+	this->priv->visible                    = FALSE;
+	this->priv->timeout                    = 5000;
+	this->priv->mouse_over                 = FALSE;
+	this->priv->distance                   = 1.0f;
+	this->priv->composited                 = gdk_screen_is_composited (
 						gtk_widget_get_screen (window));
-	this->priv->alpha                = NULL;
-	this->priv->timeline             = NULL;
-	this->priv->title_width          = 0;
-	this->priv->title_height         = 0;
-	this->priv->body_width           = 0;
-	this->priv->body_height          = 0;
-	this->priv->append               = FALSE;
-	this->priv->icon_only            = FALSE;
-	this->priv->tile_background_part = NULL;
-	this->priv->tile_background      = NULL;
-	this->priv->tile_icon            = NULL;
-	this->priv->tile_title           = NULL;
-	this->priv->tile_body            = NULL;
-	this->priv->tile_indicator       = NULL;
-	this->priv->prevent_fade         = FALSE;
+	this->priv->alpha                      = NULL;
+	this->priv->timeline                   = NULL;
+	this->priv->title_width                = 0;
+	this->priv->title_height               = 0;
+	this->priv->body_width                 = 0;
+	this->priv->body_height                = 0;
+	this->priv->append                     = FALSE;
+	this->priv->icon_only                  = FALSE;
+	this->priv->tile_background_part       = NULL;
+	this->priv->tile_background            = NULL;
+	this->priv->tile_icon                  = NULL;
+	this->priv->tile_title                 = NULL;
+	this->priv->tile_body                  = NULL;
+	this->priv->tile_indicator             = NULL;
+	this->priv->prevent_fade               = FALSE;
+	this->priv->old_icon_filename          = g_string_new ("");
 
-	update_input_shape (window, 1, 1);
+	update_input_shape (window);
 
 	return this;
 }
@@ -2317,12 +2306,17 @@ bubble_set_title (Bubble*      self,
 
 	priv = GET_PRIVATE (self);
 
-	if (priv->title)
-		g_string_free (priv->title, TRUE);
-
 	// convert any newline to space
 	text = newline_to_space (title);
 
+	if (priv->title)
+	{
+		if (g_strcmp0 (priv->title->str, text))
+			priv->title_needs_refresh = TRUE;
+
+		g_string_free (priv->title, TRUE);
+	}
+
 	priv->title = g_string_new (text);
 
 	g_object_notify (
@@ -2353,6 +2347,13 @@ bubble_set_message_body (Bubble*      self,
 
 	priv = GET_PRIVATE (self);
 
+	// filter out any HTML/markup if possible
+	text = filter_text (body);
+
+	if (priv->message_body)
+		if (g_strcmp0 (priv->message_body->str, text))
+			priv->message_body_needs_refresh = TRUE;
+
 	if (priv->message_body && priv->message_body->len != 0)
 	{
 		g_signal_emit (self,
@@ -2362,9 +2363,6 @@ bubble_set_message_body (Bubble*      self,
 		g_string_free (priv->message_body, TRUE);
 	}
 
-	/* filter out any HTML/markup if possible */
-	text = filter_text (body);
-
 	priv->message_body = g_string_new (text);
 
 	g_signal_emit (self, g_bubble_signals[MESSAGE_BODY_INSERTED], 0, text);
@@ -2395,6 +2393,15 @@ bubble_set_icon_from_path (Bubble*      self,
 
 	priv = GET_PRIVATE (self);
 
+	// check if an app tries to set the same file as icon again, this check
+	// avoids superfluous regeneration of the tile/blur-cache for the icon,
+	// thus it improves performance in update- and append-cases
+	if (!g_strcmp0 (priv->old_icon_filename->str, filepath))
+		return;
+
+	// store the new icon-basename
+	g_string_assign (priv->old_icon_filename, filepath);
+
 	if (priv->icon_pixbuf)
 	{
 		g_object_unref (priv->icon_pixbuf);
@@ -2423,6 +2430,17 @@ bubble_set_icon (Bubble*      self,
 
 	priv = GET_PRIVATE (self);
 
+	//basename = g_path_get_basename (filename);
+
+	// check if an app tries to set the same file as icon again, this check
+	// avoids superfluous regeneration of the tile/blur-cache for the icon,
+	// thus it improves performance in update- and append-cases
+	if (!g_strcmp0 (priv->old_icon_filename->str, filename))
+		return;
+
+	// store the new icon-basename
+	g_string_assign (priv->old_icon_filename, filename);
+
 	if (priv->icon_pixbuf)
 	{
 		g_object_unref (priv->icon_pixbuf);
@@ -2519,6 +2537,9 @@ bubble_set_icon_from_pixbuf (Bubble*    self,
 
 	priv = GET_PRIVATE (self);
 
+	// "reset" the stored the icon-filename, fixes LP: #451086
+	g_string_assign (priv->old_icon_filename, "\0");
+
 	if (priv->icon_pixbuf)
 	{
 		g_object_unref (priv->icon_pixbuf);
@@ -2555,14 +2576,25 @@ void
 bubble_set_value (Bubble* self,
 		  gint    value)
 {
+	BubblePrivate* priv;
+
 	if (!self || !IS_BUBBLE (self))
 		return;
 
-	GET_PRIVATE (self)->value = value;
+	priv = GET_PRIVATE (self);
 
-	g_signal_emit (self, g_bubble_signals[VALUE_CHANGED], 0, value);
-
-	_refresh_indicator (self);
+	// only really cause a refresh (blur, recreating tile-/blur-cache) if
+	// a different value has been set, this helps improve performance when
+	// a user e.g. presses and holds down keys for Volume-Up/Down or
+	// Brightness-Up/Down and apps like gnome-settings-daemon or
+	// gnome-power-daemon fire continues notification-updates due to the
+	// key-repeat events
+	if (priv->value != value)
+	{
+		priv->value = value;
+		g_signal_emit (self, g_bubble_signals[VALUE_CHANGED], 0, value);
+		_refresh_indicator (self);
+	}
 }
 
 gint
@@ -3309,6 +3341,8 @@ _calc_body_height (Bubble* self,
 void
 bubble_recalc_size (Bubble *self)
 {
+	gint           old_bubble_width  = 0;
+	gint           old_bubble_height = 0;
 	gint           new_bubble_width  = 0;
 	gint           new_bubble_height = 0;
 	Defaults*      d;
@@ -3487,11 +3521,22 @@ bubble_recalc_size (Bubble *self)
 		break;
 	}
 	priv->future_height = new_bubble_height;
+
+	bubble_get_size (self, &old_bubble_width, &old_bubble_height);
 	bubble_set_size (self, new_bubble_width, new_bubble_height);
 
-	_refresh_background (self);
-	_refresh_title (self);
-	_refresh_body (self);
+	// don't refresh tile/blur-caches for background, title or body if the
+	// size or contents have not changed, this improves performance for the
+	// update- and replace-cases
+	if (old_bubble_width != new_bubble_width ||
+	    old_bubble_height != new_bubble_height)
+		_refresh_background (self);
+
+	if (priv->title_needs_refresh)
+		_refresh_title (self);
+
+	if (priv->message_body_needs_refresh)
+		_refresh_body (self);
 
 	update_shape (self);
 }
@@ -3713,6 +3758,7 @@ bubble_append_message_body (Bubble*      self,
 	{
 		// append text to current message-body
 		g_string_append (priv->message_body, text);
+		priv->message_body_needs_refresh = TRUE;
 
 		g_signal_emit (self,
 			       g_bubble_signals[MESSAGE_BODY_INSERTED],
