@@ -276,7 +276,7 @@ draw_shadow (cairo_t* cr,
 	cairo_destroy (cr_surf);
 
 	// create and setup blur
-	blur = raico_blur_create (RAICO_BLUR_QUALITY_HIGH);
+	blur = raico_blur_create (RAICO_BLUR_QUALITY_LOW);
 	raico_blur_set_radius (blur, shadow_radius);
 
 	// now blur it
@@ -676,7 +676,7 @@ setup_tile (gint w, gint h)
 	norm_surf = copy_surface (tmp);
 	cairo_surface_destroy (tmp);
 
-	blur = raico_blur_create (RAICO_BLUR_QUALITY_HIGH);
+	blur = raico_blur_create (RAICO_BLUR_QUALITY_LOW);
 	raico_blur_set_radius (blur, 6);
 	raico_blur_apply (blur, dummy_surf);
 	raico_blur_destroy (blur);
